@@ -26,6 +26,15 @@
 
 #include "config.h" // For GLOM_ENABLE_CLIENT_ONLY
 
+
+//Avoid including the header here:
+extern "C"
+{
+typedef struct AvahiStringList AvahiStringList;
+
+typedef struct _EpcServiceInfo EpcServiceInfo;
+}
+
 namespace Glom
 {
 
@@ -53,16 +62,18 @@ public:
 #ifndef GLOM_ENABLE_CLIENT_ONLY
   void add_developer_action(const Glib::RefPtr<Gtk::Action>& refAction);
   void remove_developer_action(const Glib::RefPtr<Gtk::Action>& refAction);
+
+  void update_userlevel_ui();
 #endif // !GLOM_ENABLE_CLIENT_ONLY
 
   AppState::userlevels get_userlevel() const;
 
-#ifndef GLOM_ENABLE_CLIENT_ONLY
-  void update_userlevel_ui();
-#endif // !GLOM_ENABLE_CLIENT_ONLY
-
   void fill_menu_tables();
   void fill_menu_reports(const Glib::ustring& table_name);
+  void fill_menu_print_layouts(const Glib::ustring& table_name);
+
+  void do_menu_developer_fields(Gtk::Window& parent, const Glib::ustring table_name);
+  void do_menu_developer_relationships(Gtk::Window& parent, const Glib::ustring table_name);
 
   ///Whether to show the generated SQL queries on stdout, for debugging.
   bool get_show_sql_debug() const;
@@ -71,7 +82,7 @@ public:
   void set_show_sql_debug(bool val = true);
 
   static App_Glom* get_application();
-
+	
 protected:
   virtual void init_layout(); //override.
   virtual void init_menus_file(); //override.
@@ -80,37 +91,48 @@ protected:
   virtual void init_toolbars(); //override
   virtual void init_create_document(); //override
   virtual bool on_document_load(); //override.
+  
+  virtual void add_ui_from_string(const Glib::ustring& ui_description);
 
-  virtual bool offer_new_or_existing();
+  bool offer_new_or_existing();
 
-  virtual void on_menu_help_contents();
+  void existing_or_new_new();
+
+  void on_menu_help_contents();
 #ifndef GLOM_ENABLE_CLIENT_ONLY
-  virtual void on_menu_userlevel_developer();
-  virtual void on_menu_userlevel_operator();
-  virtual void on_menu_file_save_as_example();
-  virtual void on_menu_developer_changelanguage();
-  virtual void on_menu_developer_translations();
-  virtual void on_window_translations_hide();
+  void on_menu_userlevel_developer();
+  void on_menu_userlevel_operator();
+  void on_menu_file_save_as_example();
+  void on_menu_developer_changelanguage();
+  void on_menu_developer_translations();
+  void on_window_translations_hide();
+
+  virtual Glib::ustring ui_file_select_save(const Glib::ustring& old_file_uri); //overridden.
+  void on_userlevel_changed(AppState::userlevels userlevel);
+
+  Document_Glom* on_connection_pool_get_document();
+
+  bool recreate_database(bool& user_cancelled); //return indicates success.
+  void stop_self_hosting_of_document_database();
 #endif // !GLOM_ENABLE_CLIENT_ONLY
 
+#ifndef G_OS_WIN32
+  /** Offer a file chooser dialog, with a Browse Network button.
+   * @param browsed This will be set to true if the user chose a networked glom instance to open.
+   * @browsed_server This will be filled with the server details if browsed was set to true.
+   */
+  Glib::ustring ui_file_select_open_with_browse(bool& browsed, EpcServiceInfo*& browsed_server, Glib::ustring& browsed_service_name, const Glib::ustring& starting_folder_uri = Glib::ustring());
+#endif // !G_OS_WIN32
+
+  virtual void on_menu_file_open(); //overridden.
   virtual void on_menu_file_close(); //override.
-
-  virtual Glib::ustring ui_file_select_save(const Glib::ustring& old_file_uri); //overriden.
   virtual void document_history_add(const Glib::ustring& file_uri); //overridden.
 
-#ifndef GLOM_ENABLE_CLIENT_ONLY
-  virtual void on_userlevel_changed(AppState::userlevels userlevel);
-#endif // !GLOM_ENABLE_CLIENT_ONLY
-
   virtual Bakery::App* new_instance(); //Override
 
-#ifndef GLOM_ENABLE_CLIENT_ONLY
-  bool recreate_database(bool& user_cancelled); //return indicates success.
-#endif // !GLOM_ENABLE_CLIENT_ONLY
-
-#ifndef GLOM_ENABLE_CLIENT_ONLY
-  void stop_self_hosting_of_document_database();
-#endif // !GLOM_ENABLE_CLIENT_ONLY
+#ifndef G_OS_WIN32
+  void open_browsed_document(const EpcServiceInfo* server, const Glib::ustring& service_name);
+#endif // !G_OS_WIN32
 
   static Glib::ustring get_file_uri_without_extension(const Glib::ustring& uri);
 
@@ -128,23 +150,26 @@ protected:
 #endif // !GLOM_ENABLE_CLIENT_ONLY
 
   Gtk::VBox* m_pBoxTop;
+  Gtk::VBox* m_pBoxSidebar;
   Frame_Glom* m_pFrame;
 
 #ifndef GLOM_ENABLE_CLIENT_ONLY
   Window_Translations* m_window_translations;
+
 #endif // !GLOM_ENABLE_CLIENT_ONLY
 
-  Glib::RefPtr<Gtk::ActionGroup> m_refNavTablesActionGroup, m_refNavReportsActionGroup;
-  type_listActions m_listNavTableActions, m_listNavReportActions;
-  Gtk::UIManager::ui_merge_id m_menu_tables_ui_merge_id, m_menu_reports_ui_merge_id;
+  Glib::RefPtr<Gtk::ActionGroup> m_refNavTablesActionGroup, m_refNavReportsActionGroup, m_refNavPrintLayoutsActionGroup;
+  type_listActions m_listNavTableActions, m_listNavReportActions, m_listNavPrintLayoutActions;
+  Gtk::UIManager::ui_merge_id m_menu_tables_ui_merge_id, m_menu_reports_ui_merge_id, m_menu_print_layouts_ui_merge_id;
 
+#ifndef GLOM_ENABLE_CLIENT_ONLY
   //Set these before calling offer_saveas() (which uses ui_file_select_save()), and clear it afterwards.
   bool m_ui_save_extra_showextras;
   Glib::ustring m_ui_save_extra_title;
   Glib::ustring m_ui_save_extra_message;
 
   Glib::ustring m_ui_save_extra_newdb_title;
-#ifndef GLOM_ENABLE_CLIENT_ONLY
+
   bool m_ui_save_extra_newdb_selfhosted;
 #endif // !GLOM_ENABLE_CLIENT_ONLY
 
@@ -153,7 +178,11 @@ protected:
   // document_history_add().
   Glib::ustring m_example_uri;
 
-
+  //A temporary store for the username/password if 
+  //we already asked for them when getting the document over the network,
+  //so we can use them again when connecting directly to the database:
+  Glib::ustring m_temp_username, m_temp_password;
+	
   bool m_show_sql_debug;
 };
 
