@@ -28,16 +28,16 @@
 #include <glom/libglom/document/document_glom.h>
 
 #include "dialog_glom.h"
-
 #include "navigation/box_tables.h"
-#include "box_reports.h"
-
 #include "mode_data/notebook_data.h"
 #include "mode_find/notebook_find.h"
 
 #ifndef GLOM_ENABLE_CLIENT_ONLY
+#include "box_reports.h"
+#include "mode_design/print_layouts/box_print_layouts.h"
 #include "mode_design/dialog_fields.h"
 #include "mode_design/dialog_relationships.h"
+#include "utility_widgets/dragbar.h"
 #endif // !GLOM_ENABLE_CLIENT_ONLY
 
 #include "dialog_connection.h"
@@ -50,7 +50,9 @@ namespace Glom
 
 #ifndef GLOM_ENABLE_CLIENT_ONLY
 class Dialog_Layout_Report;
+class Window_PrintLayout_Edit;
 class Dialog_AddRelatedTable;
+class Dialog_RelationshipsOverview;
 #endif // !GLOM_ENABLE_CLIENT_ONLY
 
 class Frame_Glom :
@@ -63,18 +65,20 @@ public:
   virtual ~Frame_Glom();
 
   void set_databases_selected(const Glib::ustring& strName);
-  void on_box_tables_selected(const Glib::ustring& strName);
 
+  void on_box_tables_selected(const Glib::ustring& strName);
 #ifndef GLOM_ENABLE_CLIENT_ONLY
   void on_box_reports_selected(const Glib::ustring& strName);
-#endif // !GLOM_ENABLE_CLIENT_ONLY
+  void on_box_print_layouts_selected(const Glib::ustring& strName);
 
-#ifndef GLOM_ENABLE_CLIENT_ONLY
   void on_menu_userlevel_Developer(const Glib::RefPtr<Gtk::RadioAction>& action, const Glib::RefPtr<Gtk::RadioAction>& operator_action);
   void on_menu_userlevel_Operator(const Glib::RefPtr<Gtk::RadioAction>& action);
-#endif // !GLOM_ENABLE_CLIENT_ONLY
 
   void on_menu_file_export();
+  void on_menu_file_import();
+  void on_menu_file_print_edit_layouts();
+
+#endif // !GLOM_ENABLE_CLIENT_ONLY
   void on_menu_file_print();
 
   void on_menu_Mode_Data();
@@ -82,43 +86,48 @@ public:
 
   void on_menu_report_selected(const Glib::ustring& report_name);
 
+#ifndef GLOM_ENABLE_CLIENT_ONLY
+  void on_menu_print_layout_selected(const Glib::ustring& print_layout_name);
+#endif // !GLOM_ENABLE_CLIENT_ONLY
+
   //virtual void on_menu_Navigate_Database();
   //virtual void do_menu_Navigate_Database(bool bUseList = true);
+
+  void do_menu_Navigate_Table(bool open_default = false);
+
 #ifndef GLOM_ENABLE_CLIENT_ONLY
   void on_menu_Tables_EditTables();
   void on_menu_Tables_AddRelatedTable();
 #endif // !GLOM_ENABLE_CLIENT_ONLY
 
-  void do_menu_Navigate_Table(bool open_default = false);
-
 #ifndef GLOM_ENABLE_CLIENT_ONLY
-  void on_menu_Tables_EditReports();
-
+  void on_menu_Reports_EditReports();
+  void on_menu_File_EditPrintLayouts();
   void on_menu_developer_database_preferences();
-
   void on_menu_developer_fields();
   void do_menu_developer_fields(Gtk::Window& parent);
-
+  void do_menu_developer_fields(Gtk::Window& parent, const Glib::ustring table_name);
   void on_menu_developer_relationships_overview();
   void on_menu_developer_relationships();
+  void do_menu_developer_relationships(Gtk::Window& parent, const Glib::ustring table_name);
   void on_menu_developer_users();
   void on_menu_developer_layout();
   void on_menu_developer_reports();
+  void on_menu_developer_print_layouts();
   void on_menu_developer_script_library();
 
   void on_developer_dialog_hide();
-#endif // !GLOM_ENABLE_CLIENT_ONLY
 
-#ifndef GLOM_ENABLE_CLIENT_ONLY
   void on_dialog_layout_report_hide();
   void on_dialog_reports_hide();
-#endif // !GLOM_ENABLE_CLIENT_ONLY
-  void on_dialog_tables_hide();
-#ifndef GLOM_ENABLE_CLIENT_ONLY
+  void on_dialog_layout_print_hide();
+  void on_dialog_print_layouts_hide();
 
   void on_dialog_add_related_table_request_edit_fields();
 #endif // !GLOM_ENABLE_CLIENT_ONLY
 
+  void on_dialog_tables_hide();
+
   virtual void set_document(Document_Glom* pDocument); //View override
   virtual void load_from_document(); //View override
 
@@ -137,9 +146,9 @@ public:
 
   //Show the dialog to request the password, and check whether it works.
 #ifdef GLIBMM_EXCEPTIONS_ENABLED
-  bool connection_request_password_and_attempt();
+  bool connection_request_password_and_attempt(const Glib::ustring known_username = Glib::ustring(), const Glib::ustring& known_password = Glib::ustring());
 #else
-  bool connection_request_password_and_attempt(std::auto_ptr<ExceptionConnection>& error);
+  bool connection_request_password_and_attempt(const Glib::ustring known_username = Glib::ustring(), const Glib::ustring& known_password = Glib::ustring(), std::auto_ptr<ExceptionConnection>& error);
 #endif
 
   //Show the dialog to request the password, and choose an unused database name.
@@ -147,21 +156,30 @@ public:
 
 #ifndef GLOM_ENABLE_CLIENT_ONLY
   ///Create the database for new documents, showing the Connection dialog
-  bool create_database(const Glib::ustring& database_name, const Glib::ustring& title, bool request_password = true);
+  bool create_database(const Glib::ustring& database_name, const Glib::ustring& title);
 #endif // !GLOM_ENABLE_CLIENT_ONLY
 
-  void export_data_to_string(Glib::ustring& the_string, const FoundSet& found_set, const Document_Glom::type_mapLayoutGroupSequence& sequence);
-  void export_data_to_stream(std::ostream& the_stream, const FoundSet& found_set, const Document_Glom::type_mapLayoutGroupSequence& sequence);
+  void export_data_to_string(Glib::ustring& the_string, const FoundSet& found_set, const Document_Glom::type_list_layout_groups& sequence);
+  void export_data_to_stream(std::ostream& the_stream, const FoundSet& found_set, const Document_Glom::type_list_layout_groups& sequence);
 
   /** Show the table again. For instance, if the document has changed, or we want to display it differently.
    */
   void show_table_refresh();
 
+  ///Whether we are viewing the details or list.
+  bool get_viewing_details() const;
+
 protected:
 
   //virtual void set_document(Document_Glom* pDocument); //override
 
+  /** Show the table, possibly selecting a particular record, possibly showing that in the details tab.
+   *
+   * @param table_name The database table to show.
+   * @param primary_key_value_for_details If specified, switch to the details view, and show this record.
+   */
   void show_table(const Glib::ustring& table_name, const Gnome::Gda::Value& primary_key_value_for_details = Gnome::Gda::Value());
+    
   void show_table_title();
   void update_table_in_document_from_database();
 
@@ -179,6 +197,7 @@ protected:
   void on_notebook_find_criteria(const Glib::ustring& where_clause);
   void on_button_quickfind();
   void on_button_find_all();
+  void on_notebook_data_switch_page(GtkNotebookPage* page, guint page_num);
   void on_notebook_data_record_details_requested(const Glib::ustring& table_name, Gnome::Gda::Value primary_key_value);
   void on_userlevel_changed(AppState::userlevels userlevel);
 
@@ -192,6 +211,7 @@ protected:
   //Child widgets:
   Gtk::Label* m_pLabel_Name;
   Gtk::Label* m_pLabel_Table;
+  Gtk::HBox* m_box_footer;
   Gtk::Label* m_pLabel_Mode;
   Gtk::Label* m_pLabel_userlevel;
 
@@ -207,34 +227,31 @@ protected:
   PlaceHolder* m_pBox_Mode; //Contains e.g. design mode notebook.
 
   Box_Tables* m_pBox_Tables;
-#ifndef GLOM_ENABLE_CLIENT_ONLY
-  Box_Reports* m_pBox_Reports;
-#endif // !GLOM_ENABLE_CLIENT_ONLY
 
   Notebook_Data m_Notebook_Data;
   Notebook_Find m_Notebook_Find;
 
   //Navigation:
   Dialog_Glom* m_pDialog_Tables;
-#ifndef GLOM_ENABLE_CLIENT_ONLY
-  Dialog_Glom* m_pDialog_Reports;
-#endif // !GLOM_ENABLE_CLIENT_ONLY
 
 #ifndef GLOM_ENABLE_CLIENT_ONLY
   //Developer:
+  Dialog_Glom* m_pDialog_Reports;
+  Dialog_Layout_Report* m_pDialogLayoutReport;
+  Box_Reports* m_pBox_Reports;
+  
+  Dialog_Glom* m_pDialog_PrintLayouts;
+  Window_PrintLayout_Edit* m_pDialogLayoutPrint;
+  Box_Print_Layouts* m_pBox_PrintLayouts;
+
   Dialog_Fields* m_pDialog_Fields;
   Dialog_Relationships* m_pDialog_Relationships;
   Dialog_AddRelatedTable* m_dialog_addrelatedtable;
+  Dialog_RelationshipsOverview* m_dialog_relationships_overview;
+  
 #endif // !GLOM_ENABLE_CLIENT_ONLY
 
   Dialog_Connection* m_pDialogConnection;
-  Gtk::Dialog* m_pDialogConnectionFailed;
-
-#ifndef GLOM_ENABLE_CLIENT_ONLY
-  Dialog_Layout_Report* m_pDialogLayoutReport;
-#endif // !GLOM_ENABLE_CLIENT_ONLY
-
-  Box_Data_List_Related m_HackToFixLinkerError; //The implementation of this class does not seem to be in the library unless I do this. murrayc.
 };
 
 } //namespace Glom
