@@ -25,12 +25,15 @@ namespace Glom
 {
 
 LayoutItem_Portal::LayoutItem_Portal()
+: m_navigation_relationship_specific_main(false)
 {
 }
 
 LayoutItem_Portal::LayoutItem_Portal(const LayoutItem_Portal& src)
 : LayoutGroup(src),
-  UsesRelationship(src)
+  UsesRelationship(src),
+  m_navigation_relationship_specific_main(src.m_navigation_relationship_specific_main),
+  m_navigation_relationship_specific(src.m_navigation_relationship_specific)
 {
 }
 
@@ -49,6 +52,9 @@ LayoutItem_Portal& LayoutItem_Portal::operator=(const LayoutItem_Portal& src)
   LayoutGroup::operator=(src);
   UsesRelationship::operator=(src);
 
+  m_navigation_relationship_specific_main = src.m_navigation_relationship_specific_main;
+  m_navigation_relationship_specific = src.m_navigation_relationship_specific;
+
   return *this;
 }
 
@@ -66,9 +72,9 @@ void LayoutItem_Portal::change_related_field_item_name(const Glib::ustring& tabl
 void LayoutItem_Portal::change_field_item_name(const Glib::ustring& table_name, const Glib::ustring& field_name, const Glib::ustring& field_name_new)
 {
   //Look at each item:
-  for(LayoutGroup::type_map_items::iterator iterItem = m_map_items.begin(); iterItem != m_map_items.end(); ++iterItem)
+  for(LayoutGroup::type_list_items::iterator iterItem = m_list_items.begin(); iterItem != m_list_items.end(); ++iterItem)
   {
-    sharedptr<LayoutItem> item = iterItem->second;
+    sharedptr<LayoutItem> item = *iterItem;
     sharedptr<LayoutItem_Field> field_item = sharedptr<LayoutItem_Field>::cast_dynamic(item);
     if(field_item)
     {
@@ -92,10 +98,39 @@ void LayoutItem_Portal::change_field_item_name(const Glib::ustring& table_name,
   }
 }
 
+sharedptr<UsesRelationship> LayoutItem_Portal::get_navigation_relationship_specific(bool& main_relationship)
+{
+  main_relationship = m_navigation_relationship_specific_main;
+  if(!main_relationship)
+    return m_navigation_relationship_specific;
+  else
+    return sharedptr<UsesRelationship>();
+}
+
+sharedptr<const UsesRelationship> LayoutItem_Portal::get_navigation_relationship_specific(bool& main_relationship) const
+{
+  main_relationship = m_navigation_relationship_specific_main;
+  if(!main_relationship)
+    return m_navigation_relationship_specific;
+  else
+    return sharedptr<UsesRelationship>();
+}
+
+void LayoutItem_Portal::set_navigation_relationship_specific(bool main_relationship, const sharedptr<UsesRelationship>& relationship)
+{
+  m_navigation_relationship_specific_main = main_relationship;
+  if(!m_navigation_relationship_specific_main)
+    m_navigation_relationship_specific = relationship;
+  else
+    m_navigation_relationship_specific = sharedptr<UsesRelationship>();
+}
+
+/*
 void LayoutItem_Portal::debug(guint level) const
 {
   g_warning("LayoutItem_Portal::debug: level = %d", level);
   //LayoutGroup::debug(level);
 }
+*/
 
 } //namespace Glom
