@@ -186,14 +186,20 @@ RelatedRecord_tp_as_mapping_getitem(PyObject *self, PyObject *item)
 
         //Check whether the field exists in the table.
         //TODO_Performance: Do this without the useless Field information?
-        sharedptr<Field> field  = self_derived->m_document->get_field((*(self_derived->m_relationship))->get_to_table(), field_name);
+        sharedptr<Field> field = self_derived->m_document->get_field((*(self_derived->m_relationship))->get_to_table(), field_name);
         if(!field)
           g_warning("RelatedRecord_tp_as_mapping_getitem: field %s not found in table %s", field_name.c_str(), (*(self_derived->m_relationship))->get_to_table().c_str());
         else
         {
           //Try to get the value from the database:
           //const Glib::ustring parent_key_name;
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
           sharedptr<SharedConnection> sharedconnection = ConnectionPool::get_instance()->connect();
+#else
+          std::auto_ptr<ExceptionConnection> conn_error;
+          sharedptr<SharedConnection> sharedconnection = ConnectionPool::get_instance()->connect(conn_error);
+          // Ignore error, sharedconnection presence is checked below
+#endif
           if(sharedconnection)
           {
             Glib::RefPtr<Gnome::Gda::Connection> gda_connection = sharedconnection->get_gda_connection();
@@ -208,20 +214,28 @@ RelatedRecord_tp_as_mapping_getitem(PyObject *self, PyObject *item)
             Glib::ustring sql_query = "SELECT \"" + related_table + "\".\"" + field_name + "\" FROM \"" + related_table + "\""
               + " WHERE \"" + related_table + "\".\"" + related_key_name + "\" = " + *(self_derived->m_from_key_value_sqlized);
 
-             /* TODO: Fix linking problems
-             const App_Glom* app = App_Glom::get_application();
-             if(app && app->get_show_sql_debug())
-             {
-                try
-                {
-                  std::cout << "Debug: query_execute():  " << sql_query << std::endl;
-                }
-                catch(const Glib::Exception& ex)
-                {
-                  std::cout << "Debug: query string could not be converted to std::cout: " << ex.what() << std::endl;
-                 }
+            /* TODO: Fix linking problems
+            const App_Glom* app = App_Glom::get_application();
+            if(app && app->get_show_sql_debug())
+            {
+              try
+              {
+                std::cout << "Debug: RelatedRecord_tp_as_mapping_getitem():  " << sql_query << std::endl;
+              }
+              catch(const Glib::Exception& ex)
+              {
+                std::cout << "Debug: query string could not be converted to std::cout: " << ex.what() << std::endl;
+              }
             }*/
+
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
+            // TODO: Does this behave well if this throws an exception?
             Glib::RefPtr<Gnome::Gda::DataModel> datamodel = gda_connection->execute_select_command(sql_query);
+#else
+            std::auto_ptr<Glib::Error> error;
+            Glib::RefPtr<Gnome::Gda::DataModel> datamodel = gda_connection->execute_select_command(sql_query, error);
+            // Ignore error, datamodel return value is checked below
+#endif
             if(datamodel && datamodel->get_n_rows())
             {
               Gnome::Gda::Value value = datamodel->get_value_at(0, 0);
@@ -270,10 +284,11 @@ static PyMappingMethods RelatedRecord_tp_as_mapping = {
 static PyObject *
 RelatedRecord_generic_aggregate(PyGlomRelatedRecord* self, PyObject *args, PyObject *kwargs, const Glib::ustring& aggregate)
 {
-  static char *kwlist[] = { "field_name", 0 };
+  typedef const char* type_pch;
+  static type_pch kwlist[] = { "field_name", 0 };
   PyObject* py_field_name = 0;
 
-  if (!PyArg_ParseTupleAndKeywords(args, kwargs, "O:RelatedRecord.sum", kwlist, &py_field_name))
+  if (!PyArg_ParseTupleAndKeywords(args, kwargs, (char*)"O:RelatedRecord.sum", (char**)kwlist, &py_field_name))
     return NULL;
 
   if(!(PyString_Check(py_field_name)))
@@ -294,7 +309,13 @@ RelatedRecord_generic_aggregate(PyGlomRelatedRecord* self, PyObject *args, PyObj
     {
       //Try to get the value from the database:
       //const Glib::ustring parent_key_name;
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
       sharedptr<SharedConnection> sharedconnection = ConnectionPool::get_instance()->connect();
+#else
+      std::auto_ptr<ExceptionConnection> conn_error;
+      sharedptr<SharedConnection> sharedconnection = ConnectionPool::get_instance()->connect(conn_error);
+      // Ignore error, sharedconnection presence is checked below
+#endif
       if(sharedconnection)
       {
         Glib::RefPtr<Gnome::Gda::Connection> gda_connection = sharedconnection->get_gda_connection();
@@ -310,7 +331,14 @@ RelatedRecord_generic_aggregate(PyGlomRelatedRecord* self, PyObject *args, PyObj
           + " WHERE \"" + related_table + "\".\"" + related_key_name + "\" = " + *(self->m_from_key_value_sqlized);
 
         //std::cout << "PyGlomRelatedRecord: Executing:  " << sql_query << std::endl;
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
         Glib::RefPtr<Gnome::Gda::DataModel> datamodel = gda_connection->execute_select_command(sql_query);
+#else
+	std::auto_ptr<Glib::Error> error;
+        Glib::RefPtr<Gnome::Gda::DataModel> datamodel = gda_connection->execute_select_command(sql_query, error);
+	// Ignore error, the case that the command execution didn't return
+	//  a datamodel is handled below.
+#endif
         if(datamodel && datamodel->get_n_rows())
         {
           Gnome::Gda::Value value = datamodel->get_value_at(0, 0);
@@ -367,17 +395,17 @@ RelatedRecord_max(PyObject* self, PyObject *args, PyObject *kwargs)
 }
 
 static PyMethodDef RelatedRecord_methods[] = {
-    {"sum", (PyCFunction)RelatedRecord_sum, METH_VARARGS | METH_KEYWORDS,
-     "Add all values of the field in the related records."
+    {(char*)"sum", (PyCFunction)RelatedRecord_sum, METH_VARARGS | METH_KEYWORDS,
+     (char*)"Add all values of the field in the related records."
     },
-    {"count", (PyCFunction)RelatedRecord_count, METH_VARARGS | METH_KEYWORDS,
-     "Count all values in the field in the related records."
+    {(char*)"count", (PyCFunction)RelatedRecord_count, METH_VARARGS | METH_KEYWORDS,
+     (char*)"Count all values in the field in the related records."
     },
-    {"min", (PyCFunction)RelatedRecord_min, METH_VARARGS | METH_KEYWORDS,
-     "Minimum of all values of the field in the related records."
+    {(char*)"min", (PyCFunction)RelatedRecord_min, METH_VARARGS | METH_KEYWORDS,
+     (char*)"Minimum of all values of the field in the related records."
     },
-    {"max", (PyCFunction)RelatedRecord_max, METH_VARARGS | METH_KEYWORDS,
-     "Maximum of all values of the field in the related records."
+    {(char*)"max", (PyCFunction)RelatedRecord_max, METH_VARARGS | METH_KEYWORDS,
+     (char*)"Maximum of all values of the field in the related records."
     },
     {NULL, 0, 0, 0}  /* Sentinel */
 };
@@ -388,7 +416,7 @@ static PyMethodDef RelatedRecord_methods[] = {
 static PyTypeObject pyglom_RelatedRecordType = {
     PyObject_HEAD_INIT(NULL)
     0,                         /*ob_size*/
-    "glom.RelatedRecord",             /*tp_name*/
+    (char*)"glom.RelatedRecord",             /*tp_name*/
     sizeof(PyGlomRelatedRecord), /*tp_basicsize*/
     0,                         /*tp_itemsize*/
     (destructor)RelatedRecord_dealloc, /*tp_dealloc*/
@@ -407,7 +435,7 @@ static PyTypeObject pyglom_RelatedRecordType = {
     0,                         /*tp_setattro*/
     0,                         /*tp_as_buffer*/
     Py_TPFLAGS_DEFAULT,        /*tp_flags*/
-    "Glom objects",           /* tp_doc */
+    (char*)"Glom objects",           /* tp_doc */
     0,                  /* tp_traverse */
     0,                   /* tp_clear */
     0,                   /* tp_richcompare */
