@@ -18,21 +18,31 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include "config.h" //For VERSION.
+
 //We use Python for calculated fields.
 #include <Python.h> //Include it before anything else to avoid "_POSIX_C_SOURCE redefined".
 
 //#include <gnome.h>
 #include <gtkmm/main.h>
-#include <libgnome/gnome-init.h> // For gnome_program_init().
-#include <libgnomevfsmm/uri.h>
+#include <giomm.h>
+
+#ifndef GLOM_ENABLE_CLIENT_ONLY
 #include <gtksourceviewmm/init.h>
+#include <goocanvasmm/init.h>
+#endif // !GLOM_ENABLE_CLIENT_ONLY
 #include <glibmm/i18n.h>
 
-
-
-#include "config.h" //For VERSION.
+#ifdef GLOM_ENABLE_MAEMO
+#include <hildonmm/init.h>
+#endif
 
 #include "application.h"
+#include <glom/libglom/glade_utils.h>
+
+#ifndef G_OS_WIN32
+#include <fontconfig/fontconfig.h> //For cleanup.
+#endif
 
 namespace Glom
 {
@@ -74,36 +84,102 @@ OptionGroup::OptionGroup()
 
 } //namespace Glom
 
+extern "C" void __libc_freeres(void);
+
 int 
 main(int argc, char* argv[])
 {
+  //Force some cleanup at exit,
+  //to help valgrind to detect memory leaks:
+  atexit(__libc_freeres);
+
+  // TODO: I am not sure why, but this does not work. PYTHONPATH is set
+  // correctly according to getenv(), but python still does not look in it.
+  // For now, the installer installs all the python stuff directly into the 
+  // application directory, although I would like to move this to a python/
+  // subdirectory.
+#if 0
+#ifdef G_OS_WIN32
+  // Set PYTHONPATH to point to python/ because that's where the installer
+  // installs all the python modules into.
+  gchar* python_path = g_win32_get_package_installation_subdirectory(NULL, NULL, "python");
+  std::string current_path = Glib::getenv("PYTHONPATH");
+  if(current_path.empty()) current_path = python_path;
+  else current_path += (std::string(";") + python_path); // PATH-like variables are separated by ; on Windows because : is a valid character in paths.
+  g_free(python_path);
+  std::cout << "Setting " << current_path << ":" << std::endl;
+  std::cout << Glib::setenv("PYTHONPATH", current_path) << std::endl;
+  std::cout << getenv("PYTHONPATH") << std::endl;
+#endif
+#endif
+
+#ifdef G_OS_WIN32
+  // Add glom's bin directory to PATH so that g_spawn* finds the
+  // gspawn-win32-helper.exe helper program. The installer installs it there.
+  gchar* app_dir = g_win32_get_package_installation_subdirectory(NULL, NULL, "bin");
+  Glib::setenv("PATH", Glib::getenv("PATH") + ";" + app_dir);
+  g_free(app_dir);
+#endif
+
+#ifdef G_OS_WIN32
+  // Load translations relative to glom.exe on Windows
+  gchar* dir = g_win32_get_package_installation_subdirectory(NULL, NULL, "lib/locale");
+  bindtextdomain(GETTEXT_PACKAGE, dir);
+  g_free(dir);
+#else
   //Make this application use the current locale for _() translation:
   bindtextdomain(GETTEXT_PACKAGE, LOCALEDIR);  //LOCALEDIR is defined in the Makefile.am
+#endif
   bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
   textdomain(GETTEXT_PACKAGE);
 
   g_thread_init(NULL); //So we can use GMutex.
 
   Gnome::Gda::init("glom", VERSION, argc, argv);
-  gtksourceview::init();
+#ifdef GLOM_ENABLE_MAEMO
+  Hildon::init();
+#endif
 
   Glib::OptionContext context;
 
   Glom::OptionGroup group;
   context.set_main_group(group);
 
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
   try
+#else
+  std::auto_ptr<Glib::Error> error;
+#endif // GLIBMM_EXCEPTIONS_ENABLED
   {
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
     context.parse(argc, argv);
+#else
+    context.parse(argc, argv, error);
+#endif // GLIBMM_EXCEPTIONS_ENABLED
   }
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
   catch(const Glib::OptionError& ex)
+#else
+  if(error.get() != NULL)
+#endif
   {
-    std::cout << _("Error while parsing commmand-line options: ") << std::endl << ex.what() << std::endl;
-    std::cout << _("Use --help to see a list of available command-line options.") << std::endl;
-    return 0;
+#ifndef GLIBMM_EXCEPTIONS_ENABLED
+    const Glib::OptionError* exptr = dynamic_cast<Glib::OptionError*>(error.get());
+    if(exptr)
+    {
+      const Glib::OptionError& ex = *exptr;
+#endif // !GLIBMM_EXCEPTIONS_ENABLED
+      std::cout << _("Error while parsing command-line options: ") << std::endl << ex.what() << std::endl;
+      std::cout << _("Use --help to see a list of available command-line options.") << std::endl;
+      return 0;
+#ifndef GLIBMM_EXCEPTIONS_ENABLED
+    }
+    const Glib::Error& ex = *error.get();
+#else
   }
   catch(const Glib::Error& ex)
   {
+#endif
     std::cout << "Error: " << ex.what() << std::endl;
     return 0;
   }
@@ -119,17 +195,18 @@ main(int argc, char* argv[])
   Py_Initialize();
   PySys_SetArgv(argc, argv);
 
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
   try
+#endif
   {
-    //Initialize gnome_program, so that we can use gnome_help_display().    
-    gnome_program_init(PACKAGE, VERSION, LIBGNOME_MODULE, argc, argv,
-        GNOME_PARAM_HUMAN_READABLE_NAME, _("Glom"),
-        GNOME_PROGRAM_STANDARD_PROPERTIES, NULL);
-
-
     Gtk::Main mainInstance(argc, argv, context);
     Bakery::init();
 
+#ifndef GLOM_ENABLE_CLIENT_ONLY
+    gtksourceview::init();
+    Goocanvas::init(PACKAGE, VERSION, argc, argv ) ;
+#endif //!GLOM_ENABLE_CLIENT_ONLY
+
     //Get command-line parameters, if any:
     Glib::ustring input_uri = group.m_arg_filename;
 
@@ -145,27 +222,47 @@ main(int argc, char* argv[])
     if(!input_uri.empty())
     {
       //Get a URI (file://something) from the filepath:
-      input_uri = Gnome::Vfs::Uri::make_from_shell_arg(input_uri);
+      Glib::RefPtr<Gio::File> file = Gio::File::create_for_commandline_arg(input_uri);
+      if(file)
+        input_uri = file->get_uri(); 
       //std::cout << "URI = " << input_uri << std::endl;
     }
 
     //debugging:
     //input_uri = "file:///home/murrayc/cvs/gnome212/glom/examples/example_smallbusiness.glom";
 
-
+    bool install_complete = false;
+#ifndef GLOM_ENABLE_CLIENT_ONLY
     //Check that PostgreSQL is really available:
-    bool install_complete = Glom::ConnectionPool::check_postgres_is_available_with_warning();
+    install_complete = Glom::ConnectionPool::check_postgres_is_available_with_warning();
     if(!install_complete)
       return -1; //There is no point in going further because the most useful Glom functionality will not work without Postgres. Only a very cut-down Glom client would be useful without self-hosting.
+#endif // !GLOM_ENABLE_CLIENT_ONLY
 
     //Check that the libgda postgres provider is really available:
     install_complete = Glom::ConnectionPool::check_postgres_gda_client_is_available_with_warning();
     if(!install_complete)
       return -1; //There is no point in going further because Glom would not be able to connect to any Postgres servers.
 
+    // Postgres can't be started as root. initdb complains.
+    // So just prevent this in general. It is safer anyway.
+    if(!Glom::ConnectionPool::check_user_is_not_root())
+      return -1;
 
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
     // Main app
-    Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "window_main");
+    Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(Glom::Utils::get_glade_file_path("glom.glade"), "window_main");
+#else
+    std::auto_ptr<Gnome::Glade::XmlError> error;
+    Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(Glom::Utils::get_glade_file_path("glom.glade"), "window_main", "", error);
+    if(error.get())
+    {
+      std::cerr << "Glom: exception: \n  " << error->what() << std::endl;
+      return -1;
+    }
+#endif
+
+
     Glom::App_Glom* pApp_Glom = 0;
     refXml->get_widget_derived("window_main", pApp_Glom);
 
@@ -180,15 +277,29 @@ main(int argc, char* argv[])
     else
       delete pApp_Glom;
   }
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
+  catch(const Glib::Exception& ex)
+  {
+    //If this happens then comment out the try/catch, and let the debugger show the call stack.
+    std::cerr << "Glom: exception: \n  " << ex.what() << std::endl;
+  }
   catch(const std::exception& ex)
   {
     //If this happens then comment out the try/catch, and let the debugger show the call stack.
     std::cerr << "Glom: exception: \n  " << ex.what() << std::endl;
   }
+#endif // GLIBMM_EXCEPTIONS_ENABLED
 
   //We use python for calculated-fields:
   Py_Finalize();
 
+  //Clean up singletons:
+  Glom::ConnectionPool::delete_instance();
+
+  //These fail, probably because of previous things that are causing leaks:
+  //cairo_debug_reset_static_data(); //This crashes with _cairo_hash_table_destroy: Assertion `hash_table->live_entries == 0' failed.
+  //FcFini(); //This crashes with "FcCacheFini: Assertion `fcCacheChains[i] == ((void *)0)' failed."
+
   return 0;
 }
 
