@@ -24,6 +24,7 @@
 #include <glom/libglom/data_structure/relationship.h>
 #include <glom/libglom/data_structure/glomconversions.h>
 #include "dialog_layout_details.h"
+#include <glom/libglom/glade_utils.h>
 #include <glom/libglom/utils.h>
 #include <glom/glom_privs.h>
 #include "../xsl_utils.h"
@@ -37,6 +38,7 @@ namespace Glom
 
 Box_Data_Details::Box_Data_Details(bool bWithNavButtons /* = true */)
 : m_HBox(false, Utils::DEFAULT_SPACING_SMALL),
+  m_HBox_Sidebar (false, Utils::DEFAULT_SPACING_SMALL),
   m_Button_New(Gtk::Stock::ADD),
   m_Button_Del(Gtk::Stock::DELETE),
   m_Button_Nav_First(Gtk::Stock::GOTO_FIRST),
@@ -44,26 +46,20 @@ Box_Data_Details::Box_Data_Details(bool bWithNavButtons /* = true */)
   m_Button_Nav_Next(Gtk::Stock::GO_FORWARD),
   m_Button_Nav_Last(Gtk::Stock::GOTO_LAST),
   m_bDoNotRefreshRelated(false),
-  m_ignore_signals(true)
+  m_ignore_signals(true),
+  m_design_mode(false)
 {
   m_layout_name = "details";
 
-  add_view(&m_FlowTable); //Allow this to access the document too.
+  m_Button_New.set_tooltip_text(_("Create a new record."));
+  m_Button_Del.set_tooltip_text(_("Remove this record."));
+  m_Button_Nav_First.set_tooltip_text(_("View the first record in the list."));
+  m_Button_Nav_Prev.set_tooltip_text(_("View the previous record in the list."));
+  m_Button_Nav_Next.set_tooltip_text(_("View the next record in the list."));
+  m_Button_Nav_Last.set_tooltip_text(_("View the last record in the list."));
+  
 
-#ifndef GLOM_ENABLE_CLIENT_ONLY
-  Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "window_data_layout"); //TODO: Use a generic layout dialog?
-  if(refXml)
-  {
-    Dialog_Layout_Details* dialog = 0;
-    refXml->get_widget_derived("window_data_layout", dialog);
-    if(dialog)
-    {
-      m_pDialogLayout = dialog;
-      add_view(m_pDialogLayout); //Give it access to the document.
-      m_pDialogLayout->signal_hide().connect( sigc::mem_fun(static_cast<Box_Data&>(*this), &Box_Data::on_dialog_layout_hide) );
-    }
-  }
-#endif // !GLOM_ENABLE_CLIENT_ONLY
+  add_view(&m_FlowTable); //Allow this to access the document too.
 
   m_FlowTable.set_columns_count(1); //Sub-groups will have multiple columns (by default, there is one sub-group, with 2 columns).
   m_FlowTable.set_padding(Utils::DEFAULT_SPACING_SMALL);
@@ -78,7 +74,12 @@ Box_Data_Details::Box_Data_Details(bool bWithNavButtons /* = true */)
   m_ScrolledWindow.set_policy(Gtk::POLICY_NEVER, Gtk::POLICY_AUTOMATIC); /* Allow vertical scrolling, but never scroll horizontally. */
 #endif
   m_ScrolledWindow.set_shadow_type(Gtk::SHADOW_NONE); //SHADOW_IN is Recommended by the GNOME HIG, but looks odd.
-  pack_start(m_ScrolledWindow);
+
+#ifndef GLOM_ENABLE_CLIENT_ONLY
+  m_HBox_Sidebar.pack_start (m_Dragbar, Gtk::PACK_SHRINK);
+  m_Dragbar.hide();
+#endif
+  m_HBox_Sidebar.pack_start(m_ScrolledWindow);
   m_ScrolledWindow.add(m_FlowTable);
   // The FlowTable does not support native scrolling, so gtkmm adds it to a
   // viewport first that also has some shadow we do not want.
@@ -108,6 +109,7 @@ Box_Data_Details::Box_Data_Details(bool bWithNavButtons /* = true */)
   m_Button_Nav_Next.signal_clicked().connect(sigc::mem_fun(*this, &Box_Data_Details::on_button_nav_next));
   m_Button_Nav_Last.signal_clicked().connect(sigc::mem_fun(*this, &Box_Data_Details::on_button_nav_last));
 
+  pack_start(m_HBox_Sidebar);
   pack_start(m_HBox, Gtk::PACK_SHRINK);
 
   m_FlowTable.signal_field_edited().connect( sigc::mem_fun(*this,  &Box_Data_Details::on_flowtable_field_edited) );
@@ -133,14 +135,26 @@ Box_Data_Details::~Box_Data_Details()
   remove_view(&m_FlowTable);
 }
 
-Gnome::Gda::Value Box_Data_Details::get_primary_key_value() const
+Gnome::Gda::Value Box_Data_Details::get_primary_key_value(const Gtk::TreeModel::iterator& /* row */) const
 {
-  return m_primary_key_value;
+  return get_primary_key_value_selected();
 }
 
 void Box_Data_Details::set_primary_key_value(const Gtk::TreeModel::iterator& /* row */, const Gnome::Gda::Value& value)
 {
   m_primary_key_value = value;
+  set_found_set_from_primary_key_value();
+  //std::cout << "Box_Data_Details::set_primary_key_value(): m_primary_key_value=" << m_field_primary_key->sql(m_primary_key_value) << std::endl;
+}
+
+void Box_Data_Details::set_found_set_from_primary_key_value()
+{
+  if (!m_primary_key_value.is_null())
+  {
+    m_found_set.m_where_clause = "\"" + m_table_name + "\".\"" + m_field_primary_key->get_name() + 
+      "\" = " + m_field_primary_key->sql(m_primary_key_value);
+    //std::cout << "  DEBUG: Box_Data_Details::set_primary_key_value(): m_found_set.m_where_clause = " << m_found_set.m_where_clause << std::endl;
+  }
 }
 
 bool Box_Data_Details::init_db_details(const FoundSet& found_set, const Gnome::Gda::Value& primary_key_value)
@@ -148,16 +162,21 @@ bool Box_Data_Details::init_db_details(const FoundSet& found_set, const Gnome::G
   //std::cout << "Box_Data_Details::init_db_details(): primary_key_value=" << primary_key_value.to_string() << std::endl;
 
   m_primary_key_value = primary_key_value;
-
   m_field_primary_key = get_field_primary_key_for_table(found_set.m_table_name);
+   
+  const bool result = Box_Data::init_db_details(found_set); //Calls create_layout(), then fill_from_database()
 
-  return Box_Data::init_db_details(found_set); //Calls create_layout(), then fill_from_database()
+  //This is not used much, but we create it anyway:
+  m_found_set = found_set; //Not used much.
+  set_found_set_from_primary_key_value();
+ 
+  return result;
 }
 
 bool Box_Data_Details::refresh_data_from_database_with_primary_key(const Gnome::Gda::Value& primary_key_value)
 {
-  //std::cout << "refresh_data_from_database_with_primary_key(): primary_key_value=" << primary_key_value.to_string() << std::endl;
   m_primary_key_value = primary_key_value;
+  set_found_set_from_primary_key_value();
   return fill_from_database();
 }
 
@@ -181,14 +200,24 @@ void Box_Data_Details::create_layout()
     m_FlowTable.set_table(m_table_name); //This allows portals to get full Relationship information
 
     //This map of layout groups will also contain the field information from the database:
-    Document_Glom::type_mapLayoutGroupSequence layout_groups = get_data_layout_groups(m_layout_name);
+    Document_Glom::type_list_layout_groups layout_groups = get_data_layout_groups(m_layout_name);
 
-    for(Document_Glom::type_mapLayoutGroupSequence::const_iterator iter = layout_groups.begin(); iter != layout_groups.end(); ++iter)
+    for(Document_Glom::type_list_layout_groups::const_iterator iter = layout_groups.begin(); iter != layout_groups.end(); ++iter)
     {
-      m_FlowTable.add_layout_group(iter->second);
+      m_FlowTable.add_layout_group(*iter);
     }
   }
-
+#ifndef GLOM_ENABLE_CLIENT_ONLY
+  m_FlowTable.set_design_mode(m_design_mode);
+  if (m_design_mode)
+  {
+    m_Dragbar.show();
+  }
+  else
+  {
+    m_Dragbar.hide();
+  }
+#endif
 }
 
 bool Box_Data_Details::fill_from_database()
@@ -265,10 +294,41 @@ bool Box_Data_Details::fill_from_database()
         //Add extra possibly-non-visible columns that we need:
         sharedptr<LayoutItem_Field> layout_item = sharedptr<LayoutItem_Field>::create();
         layout_item->set_full_field_details(m_field_primary_key);
-        fieldsToGet.push_back(layout_item);
-
+        
+        //Get the primary key index, adding the primary key if necessary:
+        //TODO_Performance: Do this for create_layout() only, instead of repeating it for each refresh?:
+        int index_primary_key = -1; //Arbitrary default.
         //g_warning("primary_key name = %s", m_field_primary_key->get_name().c_str());
-        const int index_primary_key = fieldsToGet.size() - 1;
+        const type_vecLayoutFields::const_iterator iterFind = std::find_if(fieldsToGet.begin(), fieldsToGet.end(), predicate_LayoutItem_Field_IsSameField<LayoutItem_Field>(layout_item));
+        if(iterFind == fieldsToGet.end())
+        {
+          fieldsToGet.push_back(layout_item);
+          index_primary_key = fieldsToGet.size() - 1;
+        }
+        else
+        {
+          //TODO_Performance: Is there any quick way to get the index from iterFind?
+          //TODO_Performance: If not, then just use this instead of the predicate.
+          const type_vecLayoutFields::size_type count = fieldsToGet.size();
+          for(type_vecLayoutFields::size_type i = 0; i < count; ++i)
+          {
+            sharedptr<const LayoutItem_Field> element = fieldsToGet[i];
+            if(!element)
+              continue;
+            
+            if(element->get_name() != layout_item->get_name())
+              continue;
+            
+            //Compare the relationship and related relationshp:
+            sharedptr<const UsesRelationship> uses_a = layout_item;
+            sharedptr<const UsesRelationship> uses_b = element;
+            if(*uses_a == *uses_b)
+            {
+              index_primary_key = i;
+              break;
+            }
+          }
+        }
 
         const Glib::ustring query = Utils::build_sql_select_with_key(m_table_name, fieldsToGet, m_field_primary_key, m_primary_key_value);
         Glib::RefPtr<Gnome::Gda::DataModel> result;
@@ -295,7 +355,10 @@ bool Box_Data_Details::fill_from_database()
             if(!primary_key_is_empty)
             {
               if(index_primary_key < cols_count)
+              {
                 m_primary_key_value = result->get_value_at(index_primary_key, row_number);
+                set_found_set_from_primary_key_value();
+              }
             }
 
             //Get field values to show:
@@ -327,9 +390,6 @@ bool Box_Data_Details::fill_from_database()
     //fill_related();
 
     set_unstored_data(false);
-
-    fill_end();
-
   }
 
   return bResult;
@@ -337,29 +397,44 @@ bool Box_Data_Details::fill_from_database()
 
 void Box_Data_Details::on_button_new()
 {
-  if(confirm_discard_unstored_data())
+  if(!confirm_discard_unstored_data())
+    return;
+
+  //Don't try to add a record to a list with no fields.
+  if(m_FieldsShown.empty())
   {
-    if(m_field_primary_key && m_field_primary_key->get_auto_increment()) //If the primary key is an auto-increment:
+    //Warn the user that they won't see anything if there are no fields on the layout,
+    //doing an extra check:
+    Document_Glom* document = get_document();
+    if( document && !(document->get_data_layout_groups_have_any_fields(m_layout_name, m_table_name)) )
     {
-      //Just make a new record, and show it:
-      Gnome::Gda::Value primary_key_value = generate_next_auto_increment(m_table_name, m_field_primary_key->get_name()); //TODO: This should return a Gda::Value
-
-      record_new(false /* use entered field data */, primary_key_value);
-      refresh_data_from_database_with_primary_key(primary_key_value);
-    }
-    else
-    {
-      //It's not an auto-increment primary key,
-      //so just blank the fields ready for a primary key later.
-      refresh_data_from_database_blank(); //shows blank record.
+      Gtk::Window* parent_window = get_app_window();
+      if(parent_window)
+        Utils::show_ok_dialog(_("Layout Contains No Fields"), _("There are no fields on the layout, so there is no way to enter data in a new record."), *parent_window, Gtk::MESSAGE_ERROR);
     }
 
-  } //if(confirm_discard_unstored_data())
+    return;
+  }
+
+  if(m_field_primary_key && m_field_primary_key->get_auto_increment()) //If the primary key is an auto-increment:
+  {
+    //Just make a new record, and show it:
+    Gnome::Gda::Value primary_key_value = get_next_auto_increment_value(m_table_name, m_field_primary_key->get_name()); //TODO: This should return a Gda::Value
+
+    record_new(false /* use entered field data */, primary_key_value);
+    refresh_data_from_database_with_primary_key(primary_key_value);
+  }
+  else
+  {
+    //It's not an auto-increment primary key,
+    //so just blank the fields ready for a primary key later.
+    refresh_data_from_database_blank(); //shows blank record.
+  }
 }
 
 void Box_Data_Details::on_button_del()
 {
-  if( Conversions::value_is_empty(get_primary_key_value()) )
+  if( Conversions::value_is_empty(get_primary_key_value_selected()) )
   {
     //Tell user that a primary key is needed to delete a record:
     Gtk::MessageDialog dialog(Bakery::App_Gtk::util_bold_message(_("No primary key value.")), true);
@@ -422,7 +497,7 @@ void Box_Data_Details::set_entered_field_data(const Gtk::TreeModel::iterator& /*
   set_entered_field_data(field, value);
 }
 
-Gnome::Gda::Value Box_Data_Details::get_primary_key_value_selected()
+Gnome::Gda::Value Box_Data_Details::get_primary_key_value_selected() const
 {
   return m_primary_key_value;
 }
@@ -432,7 +507,7 @@ void Box_Data_Details::recalculate_fields_for_related_records(const Glib::ustrin
   m_FieldsCalculationInProgress.clear();
 
   //Check all fields in the parent table:
-  const Gnome::Gda::Value primary_key_value = get_primary_key_value();
+  const Gnome::Gda::Value primary_key_value = get_primary_key_value_selected();
   for(type_vecFields::iterator iter = m_TableFields.begin(); iter != m_TableFields.end(); ++iter)
   {
     const sharedptr<const Field> field = *iter;
@@ -526,18 +601,24 @@ Box_Data_Details::type_signal_requested_related_details Box_Data_Details::signal
 void Box_Data_Details::on_flowtable_layout_changed()
 {
   //Get new layout:
-  //Document_Glom::type_mapLayoutGroupSequence layout_groups;
-  //m_FlowTable.get_layout_groups(layout_groups);
+#if 0
+  Document_Glom::type_list_layout_groups layout_groups;
+  m_FlowTable.get_layout_groups(layout_groups);
 
   //Store it in the document:
   Document_Glom* document = get_document();
-  document->set_modified();
-  //if(document)
-  //  document->set_data_layout_groups(m_layout_name, m_table_name, layout_groups);
-
+  if(document)
+    document->set_data_layout_groups(m_layout_name, m_table_name, layout_groups);
   //Build the view again from the new layout:
+#endif
   create_layout();
 
+  //Store it in the document:
+  Document_Glom* document = get_document();
+  if(document)
+    document->set_modified (true);
+    
+    
   //And fill it with data:
   fill_from_database();
 }
@@ -572,7 +653,7 @@ void Box_Data_Details::on_flowtable_script_button_clicked(const sharedptr<const
 {
   if(layout_item)
   {
-    const Gnome::Gda::Value primary_key_value = get_primary_key_value();
+    const Gnome::Gda::Value primary_key_value = get_primary_key_value_selected();
     execute_button_script(layout_item, primary_key_value);
 
     //Refresh the view, in case the script changed any data:
@@ -600,7 +681,8 @@ void Box_Data_Details::on_flowtable_field_edited(const sharedptr<const LayoutIte
 
   Document_Glom* document = dynamic_cast<Document_Glom*>(get_document());
 
-  Gnome::Gda::Value primary_key_value = get_primary_key_value();
+  Gnome::Gda::Value primary_key_value = get_primary_key_value_selected();
+  //std::cout << "Box_Data_Details::on_flowtable_field_edited(): primary_key_value=" << primary_key_value.to_string() << std::endl;
   if(!Conversions::value_is_empty(primary_key_value)) //If there is not a primary key value:
   {
     Glib::ustring table_name;
@@ -611,7 +693,7 @@ void Box_Data_Details::on_flowtable_field_edited(const sharedptr<const LayoutIte
     {
       table_name = get_table_name();
       primary_key_field = m_field_primary_key;
-      primary_key_value = get_primary_key_value();
+      primary_key_value = get_primary_key_value_selected();
     }
     else
     {
@@ -634,12 +716,14 @@ void Box_Data_Details::on_flowtable_field_edited(const sharedptr<const LayoutIte
 
           primary_key_value = get_entered_field_data(layout_item);
 
-          const bool test = add_related_record_for_field(layout_field, relationship, primary_key_field, primary_key_value);
+          //Note: This just uses an existing record if one already exists:
+          Gnome::Gda::Value primary_key_value_used;
+          const bool test = add_related_record_for_field(layout_field, relationship, primary_key_field, primary_key_value, primary_key_value_used);
           if(!test)
             return;
 
           //Get the new primary_key_value if it has been created:
-          primary_key_value = get_entered_field_data(layout_item);
+          primary_key_value = primary_key_value_used;
 
           //Now that the related record exists, the following code to set the value of the other field in the related field can succeed.
         }
@@ -745,7 +829,7 @@ void Box_Data_Details::on_flowtable_field_edited(const sharedptr<const LayoutIte
       else
       {
         //Make a new record, and show it:
-        Gnome::Gda::Value primary_key_value = generate_next_auto_increment(m_table_name, m_field_primary_key->get_name());
+        Gnome::Gda::Value primary_key_value = get_next_auto_increment_value(m_table_name, m_field_primary_key->get_name());
 
         record_new(true /* use entered field data */, primary_key_value);
         refresh_data_from_database_with_primary_key(primary_key_value);
@@ -755,7 +839,7 @@ void Box_Data_Details::on_flowtable_field_edited(const sharedptr<const LayoutIte
     {
       //It is not auto-generated:
 
-      if(strFieldName == m_field_primary_key->get_name()) //if it is the primary key that is being edited.
+      if(m_field_primary_key && strFieldName == m_field_primary_key->get_name()) //if it is the primary key that is being edited.
       {
         if(!check_entered_value_for_uniqueness(m_table_name, layout_field, field_value, window))
         {
@@ -780,13 +864,16 @@ void Box_Data_Details::on_flowtable_field_edited(const sharedptr<const LayoutIte
         set_unstored_data(true); //Cause a warning if this is never put into the database.
       }
     }
-  } //if(get_primary_key_value().size())
+  } //if(get_primary_key_value_selected().size())
 }
 
 void Box_Data_Details::on_userlevel_changed(AppState::userlevels user_level)
 {
 #ifndef GLOM_ENABLE_CLIENT_ONLY
-  m_FlowTable.set_design_mode( user_level == AppState::USERLEVEL_DEVELOPER );
+  m_design_mode = ( user_level == AppState::USERLEVEL_DEVELOPER );    
+  m_FlowTable.set_design_mode(m_design_mode);
+	// Recreate layout to correctly set the size of empty flowtables
+	create_layout();
 #endif
 }
 
@@ -800,10 +887,10 @@ void Box_Data_Details::print_layout_group(xmlpp::Element* node_parent, const sha
   xmlpp::Element* nodeChildGroup = node_parent->add_child("group");
   nodeChildGroup->set_attribute("title", group->get_title());
 
-  LayoutGroup::type_map_const_items items = group->get_items();
-  for(LayoutGroup::type_map_const_items::const_iterator iter = items.begin(); iter != items.end(); ++iter)
+  LayoutGroup::type_list_const_items items = group->get_items();
+  for(LayoutGroup::type_list_const_items::const_iterator iter = items.begin(); iter != items.end(); ++iter)
   {
-    sharedptr<const LayoutItem> layout_item = iter->second;
+    sharedptr<const LayoutItem> layout_item = *iter;
 
     sharedptr<const LayoutGroup> pLayoutGroup = sharedptr<const LayoutGroup>::cast_dynamic(layout_item);
     if(pLayoutGroup)
@@ -880,10 +967,10 @@ void Box_Data_Details::print_layout()
     //The groups:
     xmlpp::Element* nodeParent = nodeRoot;
 
-    Document_Glom::type_mapLayoutGroupSequence layout_groups = get_data_layout_groups(m_layout_name);
-    for(Document_Glom::type_mapLayoutGroupSequence::const_iterator iter = layout_groups.begin(); iter != layout_groups.end(); ++iter)
+    Document_Glom::type_list_layout_groups layout_groups = get_data_layout_groups(m_layout_name);
+    for(Document_Glom::type_list_layout_groups::const_iterator iter = layout_groups.begin(); iter != layout_groups.end(); ++iter)
     {
-      sharedptr<const LayoutGroup> layout_group = iter->second;
+      sharedptr<const LayoutGroup> layout_group = *iter;
       print_layout_group(nodeParent, layout_group);
     }
 
@@ -891,4 +978,24 @@ void Box_Data_Details::print_layout()
   }
 }
 
+#ifndef GLOM_ENABLE_CLIENT_ONLY
+Dialog_Layout* Box_Data_Details::create_layout_dialog() const
+{
+  Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(Utils::get_glade_file_path("glom_developer.glade"), "window_data_layout"); //TODO: Use a generic layout dialog?
+  if(refXml)
+  {
+    Dialog_Layout_Details* dialog = 0;
+    refXml->get_widget_derived("window_data_layout", dialog);
+    return dialog;
+  }
+
+  return NULL;
+}
+
+void Box_Data_Details::prepare_layout_dialog(Dialog_Layout* dialog)
+{
+  dialog->set_document(m_layout_name, get_document(), m_table_name, m_FieldsShown); //TODO: Use m_TableFields?
+}
+#endif // !GLOM_ENABLE_CLIENT_ONLY
+
 } //namespace Glom
