@@ -22,6 +22,7 @@
 #include "dialog_choose_relationship.h"
 #include "../layout_item_dialogs/dialog_buttonscript.h"
 #include "../layout_item_dialogs/dialog_notebook.h"
+#include <glom/libglom/glade_utils.h>
 #include "../frame_glom.h" //For show_ok_dialog()
 //#include <libgnome/gnome-i18n.h>
 #include <bakery/App/App_Gtk.h> //For util_bold_message().
@@ -44,6 +45,7 @@ Dialog_Layout_Details::Dialog_Layout_Details(BaseObjectType* cobject, const Glib
   m_button_field_add_group(0),
   m_button_add_notebook(0),
   m_button_add_related(0),
+  m_button_add_related_calendar(0),
   m_button_add_button(0),
   m_button_add_text(0),
   m_button_add_image(0),
@@ -61,6 +63,10 @@ Dialog_Layout_Details::Dialog_Layout_Details(BaseObjectType* cobject, const Glib
   refGlade->get_widget("frame_related_table_navigation", m_box_related_navigation); 
   m_box_related_navigation->hide();
 
+  Gtk::Frame* box_calendar = 0;
+  refGlade->get_widget("frame_calendar", box_calendar); 
+  box_calendar->hide();
+
   refGlade->get_widget("label_table_name", m_label_table_name);
 
   refGlade->get_widget("treeview_fields", m_treeview_fields);
@@ -146,6 +152,9 @@ Dialog_Layout_Details::Dialog_Layout_Details(BaseObjectType* cobject, const Glib
   refGlade->get_widget("button_add_related", m_button_add_related);
   m_button_add_related->signal_clicked().connect( sigc::mem_fun(*this, &Dialog_Layout_Details::on_button_add_related) );
 
+  refGlade->get_widget("button_add_related_calendar", m_button_add_related_calendar);
+  m_button_add_related_calendar->signal_clicked().connect( sigc::mem_fun(*this, &Dialog_Layout_Details::on_button_add_related_calendar) );
+
   refGlade->get_widget("button_add_button", m_button_add_button);
   m_button_add_button->signal_clicked().connect( sigc::mem_fun(*this, &Dialog_Layout_Details::on_button_add_button) );
 
@@ -254,10 +263,10 @@ void Dialog_Layout_Details::add_group(const Gtk::TreeModel::iterator& parent, co
     rowGroup[m_model_items->m_columns.m_col_layout_item] = group_inserted;
 
     //Add the child items: (TODO_Performance: The child items are ignored/wasted because we clone them again.)
-    LayoutGroup::type_map_const_items items = group->get_items();
-    for(LayoutGroup::type_map_const_items::const_iterator iter = items.begin(); iter != items.end(); ++iter)
+    LayoutGroup::type_list_const_items items = group->get_items();
+    for(LayoutGroup::type_list_const_items::const_iterator iter = items.begin(); iter != items.end(); ++iter)
     {
-      sharedptr<const LayoutItem> item = iter->second;
+      sharedptr<const LayoutItem> item = *iter;
 
       sharedptr<const LayoutItem_Portal> portal = sharedptr<const LayoutItem_Portal>::cast_dynamic(item);
       if(portal) //If it is a portal
@@ -292,7 +301,6 @@ void Dialog_Layout_Details::set_document(const Glib::ustring& layout, Document_G
 
   Dialog_Layout::set_document(layout, document, table_name, table_fields);
 
-
   //Update the tree models from the document
   if(document)
   {
@@ -300,41 +308,28 @@ void Dialog_Layout_Details::set_document(const Glib::ustring& layout, Document_G
     m_label_table_name->set_text(table_name);
     m_entry_table_title->set_text( document->get_table_title(table_name) );
 
-    Document_Glom::type_mapLayoutGroupSequence mapGroups = document->get_data_layout_groups_plus_new_fields(layout, m_table_name);
-    document->fill_layout_field_details(m_table_name, mapGroups); //Update with full field information.
+    Document_Glom::type_list_layout_groups list_groups = document->get_data_layout_groups_plus_new_fields(layout, m_table_name);
+    document->fill_layout_field_details(m_table_name, list_groups); //Update with full field information.
 
-    //If no information is stored in the document, then start with something:
+    //If no information is stored in the document, add a default group
 
-    if(mapGroups.empty())
+    if(list_groups.empty())
     {
       sharedptr<LayoutGroup> group = sharedptr<LayoutGroup>::create();
       group->set_name("main");
-      group->m_columns_count = 2;
+      group->m_columns_count = 1;
 
-      guint field_sequence = 1; //0 means no sequence
-      for(type_vecLayoutFields::const_iterator iter = table_fields.begin(); iter != table_fields.end(); ++iter)
-      {
-        sharedptr<LayoutItem_Field> item = *iter;
-        item->m_sequence = field_sequence;
-
-        group->add_item(item, field_sequence);
-
-        ++field_sequence;
-      }
-
-      mapGroups[1] = group;
-    }
+      list_groups.push_back(group);
+    }      
 
     //Show the field layout
     //typedef std::list< Glib::ustring > type_listStrings;
 
     m_model_items->clear();
 
-    //guint field_sequence = 1; //0 means no sequence
-    //guint group_sequence = 1; //0 means no sequence
-    for(Document_Glom::type_mapLayoutGroupSequence::const_iterator iter = mapGroups.begin(); iter != mapGroups.end(); ++iter)
+    for(Document_Glom::type_list_layout_groups::const_iterator iter = list_groups.begin(); iter != list_groups.end(); ++iter)
     {
-      sharedptr<const LayoutGroup> group = iter->second;
+      sharedptr<const LayoutGroup> group = *iter;
       sharedptr<const LayoutGroup> portal = sharedptr<const LayoutItem_Portal>::cast_dynamic(group);
       if(group && !portal)
         add_group(Gtk::TreeModel::iterator() /* null == top-level */, group);
@@ -530,7 +525,7 @@ sharedptr<LayoutItem_Button> Dialog_Layout_Details::offer_button_script_edit(con
 
   try
   {
-    Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "window_button_script");
+    Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(Utils::get_glade_file_path("glom_developer.glade"), "window_button_script");
 
     Dialog_ButtonScript* dialog = 0;
     refXml->get_widget_derived("window_button_script", dialog);
@@ -565,7 +560,7 @@ sharedptr<LayoutItem_Text> Dialog_Layout_Details::offer_textobject_edit(const sh
 
   try
   {
-    Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "window_textobject");
+    Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(Utils::get_glade_file_path("glom_developer.glade"), "window_textobject");
 
     Dialog_TextObject* dialog = 0;
     refXml->get_widget_derived("window_textobject", dialog);
@@ -605,7 +600,7 @@ sharedptr<Relationship> Dialog_Layout_Details::offer_relationship_list(const sha
 
   try
   {
-    Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "dialog_choose_relationship");
+    Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(Utils::get_glade_file_path("glom_developer.glade"), "dialog_choose_relationship");
 
     Dialog_ChooseRelationship* dialog = 0;
     refXml->get_widget_derived("dialog_choose_relationship", dialog);
@@ -795,6 +790,36 @@ void Dialog_Layout_Details::on_button_add_related()
   enable_buttons();
 }
 
+void Dialog_Layout_Details::on_button_add_related_calendar()
+{
+  Gtk::TreeModel::iterator parent = get_selected_group_parent();
+
+  sharedptr<Relationship> relationship = offer_relationship_list();
+  if(relationship)
+  {
+    Gtk::TreeModel::iterator iter = append_appropriate_row();
+    if(iter)
+    {
+      Gtk::TreeModel::Row row = *iter;
+
+      sharedptr<LayoutItem_Portal> portal = sharedptr<LayoutItem_CalendarPortal>::create();
+      portal->set_relationship(relationship);
+      row[m_model_items->m_columns.m_col_layout_item] = portal;
+
+      //Scroll to, and select, the new row:
+      Glib::RefPtr<Gtk::TreeSelection> refTreeSelection = m_treeview_fields->get_selection();
+      if(refTreeSelection)
+        refTreeSelection->select(iter);
+
+      m_treeview_fields->scroll_to_row( Gtk::TreeModel::Path(iter) );
+
+      m_modified = true;
+    }
+  }
+
+  enable_buttons();
+}
+
 Gtk::TreeModel::iterator Dialog_Layout_Details::get_selected_group_parent() const
 {
   //Get the selected group, or a suitable parent group, or the first group:
@@ -1035,11 +1060,9 @@ void Dialog_Layout_Details::save_to_document()
     m_model_items->fill_sequences();
 
     //Get the groups and their fields:
-    Document_Glom::type_mapLayoutGroupSequence mapGroups;
-    guint group_sequence = 1; //0 means no sequence
+    Document_Glom::type_list_layout_groups list_groups;
 
     //Add the layout items:
-    //guint field_sequence = 1; //0 means no sequence
     for(Gtk::TreeModel::iterator iterFields = m_model_items->children().begin(); iterFields != m_model_items->children().end(); ++iterFields)
     {
       Gtk::TreeModel::Row row = *iterFields;
@@ -1050,17 +1073,15 @@ void Dialog_Layout_Details::save_to_document()
       if(layout_group && !layout_portal) //There may be top-level groups, but no top-level fields, because the fields must be in a group (so that they are in columns)
       {
         sharedptr<LayoutGroup> group = sharedptr<LayoutGroup>::create();
-        group->m_sequence = group_sequence;
         fill_group(iterFields, group);
 
-        mapGroups[group_sequence] = group;
-        ++group_sequence;
+        list_groups.push_back(group);
       }
     }
 
     if(document)
     {
-      document->set_data_layout_groups(m_layout_name, m_table_name, mapGroups);
+      document->set_data_layout_groups(m_layout_name, m_table_name, list_groups);
       m_modified = false;
     }
   }
@@ -1093,7 +1114,11 @@ void Dialog_Layout_Details::on_cell_data_name(Gtk::CellRenderer* renderer, const
       sharedptr<LayoutItem_Portal> layout_portal = sharedptr<LayoutItem_Portal>::cast_dynamic(layout_item);
       if(layout_portal)
       {
-        markup = _("Related: ") + layout_portal->get_relationship_name();
+        sharedptr<LayoutItem_CalendarPortal> layout_calendar = sharedptr<LayoutItem_CalendarPortal>::cast_dynamic(layout_portal);
+        if(layout_calendar)
+          markup = _("Related Calendar: ") + layout_portal->get_relationship_name();
+        else
+          markup = _("Related List: ") + layout_portal->get_relationship_name();
       }
       else
       {
