@@ -20,6 +20,7 @@
 
 #include "dialog_fielddefinition.h"
 #include "dialog_fieldcalculation.h"
+#include <glom/libglom/glade_utils.h>
 #include "../../box_db_table.h"
 //#include <libgnome/gnome-i18n.h>
 #include <glibmm/i18n.h>
@@ -84,7 +85,7 @@ Dialog_FieldDefinition::Dialog_FieldDefinition(BaseObjectType* cobject, const Gl
   //Get the formatting stuff:
   try
   {
-    Glib::RefPtr<Gnome::Glade::Xml> refXmlFormatting = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "box_formatting");
+    Glib::RefPtr<Gnome::Glade::Xml> refXmlFormatting = Gnome::Glade::Xml::create(Utils::get_glade_file_path("glom_developer.glade"), "box_formatting");
     refXmlFormatting->get_widget_derived("box_formatting", m_box_formatting);
   }
   catch(const Gnome::Glade::XmlError& ex)
@@ -119,7 +120,7 @@ void Dialog_FieldDefinition::set_field(const sharedptr<const Field>& field, cons
   m_table_name = table_name;  //Used for lookup combo boxes.
 
   //Set the Widgets from the field info:
-  //const Gnome::Gda::FieldAttributes& fieldInfo = field->get_field_info();
+  //const Glib::RefPtr<Gnome::Gda::Column>& fieldInfo = field->get_field_info();
 
   m_pEntry_Name->set_text(field->get_name());
   m_pCombo_Type->set_field_type( field->get_glom_type() );
@@ -200,9 +201,9 @@ void Dialog_FieldDefinition::set_field(const sharedptr<const Field>& field, cons
 
   on_check_lookup_toggled();
 
-   std::cout << "debug: dialog_fielddefinition.c:: m_pTextView_Calculation.gobj() gtype = " << G_OBJECT_TYPE_NAME(m_pTextView_Calculation->gobj()) << std::endl;
+  //std::cout << "debug: dialog_fielddefinition.c:: m_pTextView_Calculation.gobj() gtype = " << G_OBJECT_TYPE_NAME(m_pTextView_Calculation->gobj()) << std::endl;
   m_pTextView_Calculation->get_buffer()->set_text(calculation);
-  std::cout << "  debug: dialog_fielddefinition.c:: after get_buffer()" << std::endl;
+  //std::cout << "  debug: dialog_fielddefinition.c:: after get_buffer()" << std::endl;
 
   m_pEntry_Title->set_text(field->get_title());
 
@@ -221,20 +222,20 @@ sharedptr<Field> Dialog_FieldDefinition::get_field() const
 
   //Get the field info from the widgets:
 
-  Gnome::Gda::FieldAttributes fieldInfo = field->get_field_info(); //Preserve previous information.
+  Glib::RefPtr<Gnome::Gda::Column> fieldInfo = field->get_field_info(); //Preserve previous information.
 
-  fieldInfo.set_name(m_pEntry_Name->get_text());
+  fieldInfo->set_name(m_pEntry_Name->get_text());
 
-  fieldInfo.set_gdatype( Field::get_gda_type_for_glom_type( m_pCombo_Type->get_field_type() ) );
+  fieldInfo->set_g_type( Field::get_gda_type_for_glom_type( m_pCombo_Type->get_field_type() ) );
 
-  fieldInfo.set_unique_key(m_pCheck_Unique->get_active());
-  fieldInfo.set_primary_key(m_pCheck_PrimaryKey->get_active());
-  fieldInfo.set_auto_increment(m_pCheck_AutoIncrement->get_active());
+  fieldInfo->set_unique_key(m_pCheck_Unique->get_active());
+  fieldInfo->set_primary_key(m_pCheck_PrimaryKey->get_active());
+  fieldInfo->set_auto_increment(m_pCheck_AutoIncrement->get_active());
 
-  if(!fieldInfo.get_auto_increment()) //Ignore default_values for auto_increment fields - it's just some obscure postgres code.
+  if(!fieldInfo->get_auto_increment()) //Ignore default_values for auto_increment fields - it's just some obscure postgres code.
   {
     //Simple default value:
-    fieldInfo.set_default_value( m_pDataWidget_DefaultValueSimple->get_value() );
+    fieldInfo->set_default_value( m_pDataWidget_DefaultValueSimple->get_value() );
   }
 
   //Lookup:
@@ -254,7 +255,7 @@ sharedptr<Field> Dialog_FieldDefinition::get_field() const
   if(m_pRadio_Calculate)
     field->set_calculation(m_pTextView_Calculation->get_buffer()->get_text());
 
-  Gnome::Gda::FieldAttributes field_info_copy = fieldInfo;
+  Glib::RefPtr<Gnome::Gda::Column> field_info_copy = fieldInfo;
 
   field->set_field_info(fieldInfo);
 
@@ -367,7 +368,7 @@ void Dialog_FieldDefinition::on_combo_lookup_relationship_changed()
 void Dialog_FieldDefinition::on_button_edit_calculation()
 {
   //TODO: Share a global instance, to make this quicker?
-  Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "window_field_calculation");
+  Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom_developer.glade", "window_field_calculation");
   if(refXml)
   {
     Dialog_FieldCalculation* dialog = 0;
