@@ -23,6 +23,7 @@
 
 #include <gtkmm.h>
 #include "layoutwidgetbase.h"
+#include "layoutwidgetutils.h"
 #include <glom/libglom/data_structure/layout/layoutitem_button.h>
 #include <libglademm.h>
 
@@ -32,11 +33,10 @@ namespace Glom
 class App_Glom;
 
 class LabelGlom
-: public Gtk::Label,
-  public LayoutWidgetBase
+: public Gtk::EventBox,
+  public LayoutWidgetUtils
 {
 public:
-  explicit LabelGlom(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade);
   explicit LabelGlom();
   explicit LabelGlom(const Glib::ustring& label, float xalign, float yalign, bool mnemonic = false);
   virtual ~LabelGlom();
@@ -45,6 +45,12 @@ protected:
   void init();
 
   virtual App_Glom* get_application();
+    
+  Gtk::Label m_label;
+    
+  virtual bool on_button_press_event(GdkEventButton *event);
+  virtual void on_menu_properties_activate();
+    
 };
 
 } //namespace Glom
